@@ -1,6 +1,6 @@
 package Math::SigFigs;
 
-# Copyright (c) 1995-1999 Sullivan Beck. All rights reserved.
+# Copyright (c) 1995-2005 Sullivan Beck. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -12,18 +12,30 @@ package Math::SigFigs;
 #    Sullivan Beck (sbeck@cpan.org)
 # Any suggestions, bug reports, or donations :-) should be sent to me.
 
-# Version 1.00  12/05/96
+# Version 1.00  1996-12-05
 #    Initial creation
 #
-# Version 1.01  01/28/97
+# Version 1.01  1997-01-28
 #    Used croak and changed die's to confess.
 #    "101" is now returned as "101." .
 #    Fixed where 9.99 wasn't being correctly returned with 1 sigfig.
 #       Kyle Krom <kromk@pt.Cyanamid.COM>
 #
-# Version 1.02
+# Version 1.02  2000-01-10
 #    Fixed where 1249.01 wasn't correctly rounded to 1200.
 #       Janna Wemekamp <jwemekam@erin.gov.au>
+#
+# Version 1.03  2003-09-11
+#    Fixed a bug where I left off the sign.  Steve Reaser
+#       <steve_reaser@webassign.net>
+#    Fixed a bug in subSF where numbers ending in zero were truncated.
+#       Andrew Grall <AGrall@dcds.edu>
+#
+# Version 1.04  2005-06-30
+#    Complete rewrite of addSF.
+#      - stopped using sprintf (which does not return the same results on
+#        all platforms.
+#      - replaced IsReal with Simplify.
 
 ########################################################################
 
@@ -34,84 +46,117 @@ use Carp;
 @EXPORT = qw(FormatSigFigs
              CountSigFigs
 );
-@EXPORT_OK = qw(FormatSigFigs CountSigFigs addSF subSF multSF divSF);
-%EXPORT_TAGS = (all => \@EXPORT_OK);
+@EXPORT_STD   = qw(FormatSigFigs CountSigFigs addSF subSF multSF divSF
+                   VERSION);
+@EXPORT_DEBUG = qw(LSP Simplify);
+@EXPORT_OK = (@EXPORT_STD, @EXPORT_DEBUG);
+
+%EXPORT_TAGS = (all => \@EXPORT_STD, debug => \@EXPORT_DEBUG);
+
+$VERSION = 1.04;
 
 use strict;
 
 sub addSF {
   my($n1,$n2)=@_;
-  return ()  if (! &IsReal($n1)  or  ! &IsReal($n2));
+  $n1 = Simplify($n1);
+  $n2 = Simplify($n2);
+  return ()  if (! defined $n1  ||  ! defined $n2);
   return $n2 if ($n1==0);
   return $n1 if ($n2==0);
-  my($m1,$m2,$m,$n)=();
-  if ($n1 =~ /\.(.*)$/) {
-    $m1=length($1);
-  } else {
-    $n1 =~ /(0*)$/;
-    $m1=-length($1);
-  }
-  if ($n2 =~ /\.(.*)$/) {
-    $m2=length($1);
-  } else {
-    $n2 =~ /(0*)$/;
-    $m2=-length($1);
-  }
-  $m=($m1<$m2 ? $m1 : $m2);
-  $n=$n1+$n2;
-  return 0  if ($n==0);
 
-  if ($n=~ /[+-]?(.*)\.(.*)/) {           # 123.456
-    ($m1,$m2)=(length($1),length($2));
-    if ($m<0) {                           # 120
-      $n=&FormatSigFigs($n,$m1+$m);
-    } elsif ($m>=0) {                     # 123. 123.5
-      $n=sprintf("%.$m"."f",$n);
-      $n .= "."  if ($m==0);
+  my $m1 = LSP($n1);
+  my $m2 = LSP($n2);
+  my $m  = ($m1>$m2 ? $m1 : $m2);
+
+  my($n) = $n1+$n2;
+  my($s) = ($n<0 ? "-" : "");
+  $n     = -1*$n  if ($n<0);          # n = 1234.44           5678.99
+  $n =~ /^(\d*)/;
+  my $i = ($1);                       # i = 1234              5678
+  my $l = length($i);                 # l = 4
+
+  if ($m>0) {                         # m = 5,4,3,2,1
+    if ($l >= $m+1) {                 # m = 3,2,1; l-m = 1,2,3
+      $n = FormatSigFigs($n,$l-$m);   # n = 1000,1200,1230    6000,5700,5680
+    } elsif ($l == $m) {              # m = 4
+      if ($i =~ /^[5-9]/) {
+        $n = 1 . "0"x$m;              # n =                   10000
+      } else {
+        return 0;                     # n = 0
+      }
+    } else {                          # m = 5
+      return 0;
     }
 
-  } else {                                # 12340
-    if ($m<0) {                           # 12300
-      $n=&FormatSigFigs($n,length($n)+$m);
-    } elsif ($m>=0) {                     # 12340. 12340.0
-      $n .= "." . "0"x$m;
+  } elsif ($i>0) {                    # n = 1234.44           5678.99
+    $n = FormatSigFigs($n,$l-$m);     # m = 0,-1,-2,...
+
+  } else {                            # n = 0.1234    0.00123   0.00567
+    $n =~ /\.(0*)(\d+)/;
+    my ($z,$d) = ($1,$2);
+    $m = -$m;
+
+    if ($m > length($z)) {            # m = -1,-2,..  -3,-4,..  -3,-4,..
+      $n = FormatSigFigs($n,$m-length($z));
+
+    } elsif ($m == length($z)) {      # m =           -2        -2
+      if ($d =~ /^[5-9]/) {
+        $n = "0." . "0"x($m-1) . "1"; # n =                     0.01
+      } else {
+        return 0;                     # n =           0
+      }
+
+    } else {                          # m =           -1        -1
+      return 0;
     }
   }
-  $n;
+
+  return "$s$n";
 }
 
 sub subSF {
   my($n1,$n2)=@_;
-  &addSF($n1,-$n2);
+  if ($n2<0) {
+    $n2 =~ s/\-//;
+  } else {
+    $n2 =~ s/^\+?/-/;
+  }
+  addSF($n1,$n2);
 }
 
 sub multSF {
   my($n1,$n2)=@_;
-  return ()  if (! &IsReal($n1)  or  ! &IsReal($n2));
+  $n1 = Simplify($n1);
+  $n2 = Simplify($n2);
+  return ()  if (! defined $n1  ||  ! defined $n2);
   return 0   if ($n1==0  or  $n2==0);
-  my($m1)=&CountSigFigs($n1);
-  my($m2)=&CountSigFigs($n2);
+  my($m1)=CountSigFigs($n1);
+  my($m2)=CountSigFigs($n2);
   my($m)=($m1<$m2 ? $m1 : $m2);
   my($n)=$n1*$n2;
-  &FormatSigFigs($n,$m);
+  FormatSigFigs($n,$m);
 }
 
 sub divSF {
   my($n1,$n2)=@_;
-  return ()  if (! &IsReal($n1)  or  ! &IsReal($n2));
+  $n1 = Simplify($n1);
+  $n2 = Simplify($n2);
+  return ()  if (! defined $n1  ||  ! defined $n2);
   return 0   if ($n1==0);
   return ()  if ($n2==0);
-  my($m1)=&CountSigFigs($n1);
-  my($m2)=&CountSigFigs($n2);
+  my($m1)=CountSigFigs($n1);
+  my($m2)=CountSigFigs($n2);
   my($m)=($m1<$m2 ? $m1 : $m2);
   my($n)=$n1/$n2;
-  &FormatSigFigs($n,$m);
+  FormatSigFigs($n,$m);
 }
 
 sub FormatSigFigs {
   my($N,$n)=@_;
   my($ret);
-  return ""  if (! &IsReal($N)  or  ! &IsInt($n)  or  $n<1);
+  $N = Simplify($N);
+  return ""  if (! defined($N)  or  $n !~ /^\d+$/  or  $n<1);
   my($l,$l1,$l2,$m,$s)=();
   $N=~ s/\s+//g;               # Remove all spaces
   $N=~ s/^([+-]?)//;           # Remove sign
@@ -122,7 +167,7 @@ sub FormatSigFigs {
   $N= "0$N"  if ($N=~ /^\./);  # Turn .2 into 0.2
 
   # If $N has fewer sigfigs than requested, pad it with zeros and return it.
-  $m=&CountSigFigs($N);
+  $m=CountSigFigs($N);
   if ($m==$n) {
     $N="$N."  if (length($N)==$n);
     return "$s$N";
@@ -143,7 +188,7 @@ sub FormatSigFigs {
       $l=$n-$l1;
       ($l2>$l) && ($N=~ s/5$/6/);         # 4.95 rounds down... make it go up
       $ret=sprintf("%.${l}f",$N);
-      $m=&CountSigFigs($ret);
+      $m=CountSigFigs($ret);
       if ($m==$n) {
         $ret="$ret."  if ($l==0 && $m==length($ret));
         return "$s$ret";
@@ -166,7 +211,7 @@ sub FormatSigFigs {
       $N="$a.$b$c";
       $N=sprintf("%.0f",$N);              # Turn it to 123
       $N .= "0" x length($b);             # Turn it to 1230
-      return $N;
+      return "$s$N";
     }
 
   } elsif ($N=~ /^0\.(0*)(\d*)$/) {       # 0.0123
@@ -174,7 +219,7 @@ sub FormatSigFigs {
     ($l2>$n) && ($N=~ s/5$/6/);
     $l=$l1+$n;
     $ret=sprintf("%.${l}f",$N);
-    $m=&CountSigFigs($ret);
+    $m=CountSigFigs($ret);
     return "$s$ret"  if ($n==$m);
 
     # special cases 0.099 (1) -> 0.1
@@ -182,7 +227,7 @@ sub FormatSigFigs {
 
     $l--;
     $ret=sprintf("%.${l}f",$N);
-    $m=&CountSigFigs($ret);
+    $m=CountSigFigs($ret);
     $ret="$ret."  if ($l==0);
     return "$s$ret"  if ($n==$m);
     $ret =~ s/0$//;
@@ -213,7 +258,8 @@ sub FormatSigFigs {
 
 sub CountSigFigs {
   my($N)=@_;
-  return ()  if (! &IsReal($N));
+  $N = Simplify($N);
+  return ()  if (! defined($N));
   return 0   if ($N==0);
 
   my($tmp)=();
@@ -231,42 +277,36 @@ sub CountSigFigs {
 
 ########################################################################
 # NOT FOR EXPORT
+#
+# These are exported above only for debug purposes.  They are not
+# for general use.  They are not guaranteed to remain backward
+# compatible (or even to exist at all) in future versions.
 ########################################################################
 
-sub IsReal {
-  my($N,$low,$high)=@_;
-  return 0 if ($N eq "");
-
-  my($sign)='^\s* [-+]? \s*';
-  my($int) ='\d+';
-  my($dec) ='(\.\d*)? \s* $ ';
-  my($Dec) =' \.\d*   \s* $ ';
-
-  if ($N =~ /$sign $int $dec/x or
-      $N =~ /$sign $Dec/x) {
-    if (defined $low  and  defined $high) {
-      $N=~ s/\s+//;
-      return 1  if ($N>=$low  and  $N<=$high);
-      return 0;
-    }
-    return 1;
+# This returns the power of the least sigificant digit.
+sub LSP {
+  my($n) = @_;
+  $n =~ s/\-//;
+  if ($n =~ /(.*)\.(.+)/) {
+    return -length($2);
+  } elsif ($n =~ /\.$/) {
+    return 0;
+  } else {
+    return length($n) - CountSigFigs($n);
   }
-  return 0;
 }
 
-sub IsInt {
-  my($N,$low,$high)=@_;
-  return 0 if ($N eq "");
-  my($sign)='^\s* [-+]? \s*';
-  my($int) ='\d+ \s* $ ';
-  if ($N =~ /$sign $int/x) {
-    if (defined $low  and  defined $high) {
-      return 1  if ($N>=$low  and  $N<=$high);
-      return 0;
-    }
-    return 1;
-  }
-  return 0;
+# This prepares a number by converting it to it's simplest correct
+# form.
+sub Simplify {
+  my($n)    = @_;
+  return undef  if (! defined $n);
+  return undef  if ($n !~ /^\s*([+-]?)\s*0*(\d+\.?\d*)\s*$/  and
+                    $n !~ /^\s*([+-]?)\s*0*(\.\d+)\s*$/);
+  $n="$1$2";
+  return 0  if ($n==0);
+  $n=~ s/\+//;
+  return $n;
 }
 
 1;
@@ -291,15 +331,15 @@ form.  If you are going to be doing arithmetic, use the second line.
 
 The following routines do simple counting/formatting:
 
-  $n=&CountSigFigs($num);
-  $num=&FormatSigFigs($num,$n);
+  $n=CountSigFigs($num);
+  $num=FormatSigFigs($num,$n);
 
 Use the following routines to do arithmetic operations.
 
-  $num=&addSF($n1,$n2);
-  $num=&subSF($n1,$n2);
-  $num=&multSF($n1,$n2);
-  $num=&divSF($n1,$n2);
+  $num=addSF($n1,$n2);
+  $num=subSF($n1,$n2);
+  $num=multSF($n1,$n2);
+  $num=divSF($n1,$n2);
 
 =head1 DESCRIPTION
 
@@ -320,7 +360,7 @@ given number of significant figures, and do basic arithmetic.
 
 =item CountSigFigs
 
-  $n=&CountSigFigs($N);
+  $n=CountSigFigs($N);
 
 This returns the number of significant figures in a number.  It returns
 () if $N is not a number.
@@ -337,7 +377,7 @@ This returns the number of significant figures in a number.  It returns
 
 =item FormatSigFigs
 
-  $str=&FormatSigFigs($N,$n)
+  $str=FormatSigFigs($N,$n)
 
 This returns a string containing $N formatted to $n significant figures.
 This will work for all cases except something like "2400" formatted to
@@ -378,7 +418,7 @@ These routines do not work with scientific notation (exponents).  As a
 result, it is impossible to unambiguously format some numbers.  For
 example,
 
-  $str=&FormatSigFigs("2400",3);
+  $str=FormatSigFigs("2400",3);
 
 will by necessity return the string "2400" which does NOT have 3
 significant figures.  This is not a bug.  It is simply a fundamental
